@@ -1,4 +1,4 @@
-/* $Id: extern.h,v 1.1.1.1 1999/09/26 06:32:41 hpa Exp $ */
+/* $Id: extern.h,v 1.2 2001/07/21 02:02:43 hpa Exp $ */
 
 /* $OpenBSD: extern.h,v 1.2 1996/06/26 05:40:33 deraadt Exp $	*/
 /* $NetBSD: extern.h,v 1.2 1994/12/08 09:51:24 jtc Exp $	*/
@@ -38,5 +38,10 @@
  *	@(#)extern.h	8.1 (Berkeley) 6/6/93
  */
 
+#ifndef RECVFILE_H
+#define RECVFILE_H
+
 void	recvfile (int, char *, char *);
 void	sendfile (int, char *, char *);
+
+#endif
