@@ -1,4 +1,4 @@
-/* $Id: tftp.c,v 1.5 2001/08/03 03:17:57 hpa Exp $ */
+/* $Id: tftp.c,v 1.6 2001/08/10 18:25:08 hpa Exp $ */
 
 /* $OpenBSD: tftp.c,v 1.4 1997/08/06 06:43:45 deraadt Exp $	*/
 /* $NetBSD: tftp.c,v 1.5 1995/04/29 05:55:25 cgd Exp $	*/
@@ -42,7 +42,7 @@
 /* static char sccsid[] = "@(#)tftp.c	8.1 (Berkeley) 6/6/93"; */
 /* static char rcsid[] = "$OpenBSD: tftp.c,v 1.4 1997/08/06 06:43:45 deraadt Exp $"; */
 static const char *rcsid UNUSED =
-"tftp-hpa $Id: tftp.c,v 1.5 2001/08/03 03:17:57 hpa Exp $";
+"tftp-hpa $Id: tftp.c,v 1.6 2001/08/10 18:25:08 hpa Exp $";
 #endif /* not lint */
 
 /* Many bug fixes are from Jim Guyton <guyton@rand-unix> */
@@ -336,7 +336,7 @@ makerequest(int request, const char *name,
 
 struct errmsg {
 	int	e_code;
-	char	*e_msg;
+	const char *e_msg;
 } errmsgs[] = {
 	{ EUNDEF,	"Undefined error code" },
 	{ ENOTFOUND,	"File not found" },
@@ -385,7 +385,7 @@ nak(int error)
 static void
 tpacket(const char *s, struct tftphdr *tp, int n)
 {
-	static char *opcodes[] =
+	static const char *opcodes[] =
 	   { "#0", "RRQ", "WRQ", "DATA", "ACK", "ERROR", "OACK" };
 	char *cp, *file;
 	u_short op = ntohs(tp->th_opcode);
