@@ -1,4 +1,4 @@
-/* $Id: tftpsubs.h,v 1.6 2001/08/03 03:17:57 hpa Exp $ */
+/* $Id: tftpsubs.h,v 1.7 2001/08/09 06:19:00 hpa Exp $ */
 
 /* $OpenBSD: tftpsubs.h,v 1.2 1996/06/26 05:40:37 deraadt Exp $	*/
 /* $NetBSD: tftpsubs.h,v 1.2 1994/12/08 09:51:32 jtc Exp $	*/
@@ -51,6 +51,10 @@
 #include <setjmp.h>
 #include "../config.h"
 
+#ifndef HAVE_IPPORT_TFTP_DEFINITION
+#define IPPORT_TFTP	69
+#endif
+
 struct tftphdr;
 
 struct tftphdr *r_init(void);
