@@ -1,4 +1,4 @@
-/* $Id: remap.c,v 1.4 2001/07/26 22:36:14 hpa Exp $ */
+/* $Id: remap.c,v 1.6 2001/08/10 18:25:08 hpa Exp $ */
 /* ----------------------------------------------------------------------- *
  *   
  *   Copyright 2001 H. Peter Anvin - All Rights Reserved
@@ -42,7 +42,7 @@ struct rule {
   int nrule;
   int rule_flags;
   regex_t rx;
-  char *pattern;
+  const char *pattern;
 };
 
 /* Do \-substitution.  Call with string == NULL to get length only. */
@@ -239,6 +239,26 @@ struct rule *parserulefile(FILE *f)
   return first_rule;
 }
 
+/* Destroy a rule file data structure */
+void freerules(struct rule *r)
+{
+  struct rule *next;
+
+  while ( r ) {
+    next = r->next;
+
+    regfree(&r->rx);
+
+    /* "" patterns aren't allocated by malloc() */
+    if ( r->pattern && *r->pattern )
+      free((void *)r->pattern);
+  
+    free(r);
+
+    r = next;
+  }
+}
+
 /* Execute a rule set on a string; returns a malloc'd new string. */
 char *rewrite_string(const char *input, const struct rule *rules, int is_put)
 {
