@@ -1,4 +1,4 @@
-/* $Id: remap.h,v 1.3 2001/07/20 06:59:54 hpa Exp $ */
+/* $Id: remap.h,v 1.4 2001/08/10 18:07:46 hpa Exp $ */
 /* ----------------------------------------------------------------------- *
  *   
  *   Copyright 2001 H. Peter Anvin - All Rights Reserved
@@ -30,6 +30,9 @@ struct rule;
 /* Read a rule file */
 struct rule *parserulefile(FILE *);
 
+/* Destroy a rule file data structure */
+void freerules(struct rule *);
+
 /* Execute a rule set on a string; returns a malloc'd new string. */
 char *rewrite_string(const char *, const struct rule *, int);
 
