@@ -1,4 +1,4 @@
-/* tftp-hpa: $Id: tftpd.c,v 1.23 2001/08/03 03:17:57 hpa Exp $ */
+/* tftp-hpa: $Id: tftpd.c,v 1.30 2001/08/10 18:47:11 hpa Exp $ */
 
 /* $OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $	*/
 
@@ -44,7 +44,7 @@ static const char *copyright UNUSED =
 /*static char sccsid[] = "from: @(#)tftpd.c	5.13 (Berkeley) 2/26/91";*/
 /*static char rcsid[] = "$OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $: tftpd.c,v 1.6 1997/02/16 23:49:21 deraadt Exp $";*/
 static const char *rcsid UNUSED =
-"tftp-hpa $Id: tftpd.c,v 1.23 2001/08/03 03:17:57 hpa Exp $";
+"tftp-hpa $Id: tftpd.c,v 1.30 2001/08/10 18:47:11 hpa Exp $";
 #endif /* not lint */
 
 /*
@@ -122,7 +122,7 @@ off_t	tsize;
 int     tsize_ok;
 
 int	ndirs;
-char	**dirs;
+const char **dirs;
 
 int	secure = 0;
 int	cancreate = 0;
@@ -130,7 +130,9 @@ int	cancreate = 0;
 int verbosity = 0;
 
 struct formats;
+#ifdef WITH_REGEX
 static struct rule *rewrite_rules = NULL;
+#endif
 
 int tftp(struct tftphdr *, int);
 void nak(int);
@@ -144,7 +146,7 @@ int set_tsize(char *, char **);
 int set_timeout(char *, char **);
 
 struct options {
-  char    *o_opt;
+  const char    *o_opt;
   int     (*o_fnc)(char *, char **);
 } options[] = {
   { "blksize",    set_blksize  },
@@ -166,11 +168,31 @@ static void handle_sighup(int sig)
 static void
 usage(void)
 {
-  syslog(LOG_ERR, "Usage: %s [-vc][-m mappings][-u user][-t timeout][-r option...] [-s] [directory ...]",
+  syslog(LOG_ERR, "Usage: %s [-vcl][-a address][-m mappings][-u user][-t timeout][-r option...] [-s] [directory ...]",
 	 __progname);
   exit(EX_USAGE);
 }
 
+
+#ifdef WITH_REGEX
+static struct rule *
+read_remap_rules(const char *file)
+{
+  FILE *f;
+  struct rule *rulep;
+
+  f = fopen(file, "rt");
+  if ( !f ) {
+    syslog(LOG_ERR, "Cannot open map file: %s: %m", file);
+    exit(EX_NOINPUT);
+  }
+  rulep = parserulefile(f);
+  fclose(f);
+
+  return rulep;
+}
+#endif
+
 int
 main(int argc, char **argv)
 {
@@ -178,20 +200,26 @@ main(int argc, char **argv)
   struct passwd *pw;
   struct options *opt;
   struct sockaddr_in myaddr;
-  int n = 0;
+  struct sockaddr_in bindaddr;
+  int n;
   int on = 1;
   int fd = 0;
+  int listen = 0;		/* Standalone (listen) mode */
+  char *address = NULL;		/* Address to listen to */
   int pid;
   int c;
   int setrv;
-  int timeout = 900;	/* Default timeout */
-  char *user = "nobody";	/* Default user */
-  
+  int timeout = 900;		/* Default timeout */
+  const char *user = "nobody";	/* Default user */
+#ifdef WITH_REGEX
+  char *rewrite_file = NULL;
+#endif
+
   __progname = basename(argv[0]);
   
   openlog(__progname, LOG_PID | LOG_NDELAY, LOG_DAEMON);
   
-  while ((c = getopt(argc, argv, "csvu:r:t:m:")) != -1)
+  while ((c = getopt(argc, argv, "csvla:u:r:t:m:")) != -1)
     switch (c) {
     case 'c':
       cancreate = 1;
@@ -199,6 +227,12 @@ main(int argc, char **argv)
     case 's':
       secure = 1;
       break;
+    case 'l':
+      listen = 1;
+      break;
+    case 'a':
+      address = optarg;
+      break;
     case 't':
       timeout = atoi(optarg);
       break;
@@ -219,20 +253,11 @@ main(int argc, char **argv)
       break;
 #ifdef WITH_REGEX
     case 'm':
-      {
-	FILE *f;
-	if ( rewrite_rules ) {
-	  syslog(LOG_ERR, "Multiple -m options");
-	  exit(EX_USAGE);
-	}
-	f = fopen(optarg, "rt");
-	if ( !f ) {
-	  syslog(LOG_ERR, "Cannot open map file: %s: %m", optarg);
-	  exit(EX_NOINPUT);
-	}
-	rewrite_rules = parserulefile(f);
-	fclose(f);
+      if ( rewrite_file ) {
+	syslog(LOG_ERR, "Multiple -m options");
+	exit(EX_USAGE);
       }
+      rewrite_file = optarg;
       break;
 #endif
     case 'v':
@@ -242,20 +267,12 @@ main(int argc, char **argv)
       usage();
       break;
     }
-  
-  for (; optind != argc; optind++) {
-    if (dirs)
-      dirs = realloc(dirs, (ndirs+2) * sizeof (char *));
-    else
-      dirs = calloc(ndirs+2, sizeof(char *));
-    if (dirs == NULL) {
-      syslog(LOG_ERR, "malloc: %m");
-      exit(EX_OSERR);
-    }			
-    dirs[n++] = argv[optind];
-    dirs[n] = NULL;
-    ndirs++;
-  }
+
+  dirs = xmalloc((argc-optind+1)*sizeof(char *));
+  for ( ndirs = 0 ; optind != argc ; optind++ )
+    dirs[ndirs++] = argv[optind];
+
+  dirs[ndirs] = NULL;
   
   if (secure) {
     if (ndirs == 0) {
@@ -282,7 +299,81 @@ main(int argc, char **argv)
     syslog(LOG_ERR, "ioctl(FIONBIO): %m");
     exit(EX_OSERR);
   }
-  
+
+#ifdef WITH_REGEX
+  if ( rewrite_file )
+    rewrite_rules = read_remap_rules(rewrite_file);
+#endif
+
+  /* If we're running standalone, set up the input port */
+  if ( listen ) {
+    fd = socket(PF_INET, SOCK_DGRAM, 0);
+    
+    memset(&bindaddr, 0, sizeof bindaddr);
+    bindaddr.sin_addr.s_addr = INADDR_ANY;
+    bindaddr.sin_port = htons(IPPORT_TFTP);
+
+    if ( address ) {
+      char *portptr, *eportptr;
+      struct hostent *hostent;
+      struct servent *servent;
+      unsigned long port;
+
+      address = tfstrdup(address);
+      portptr = strrchr(address, ':');
+      if ( portptr )
+	*portptr++ = '\0';
+      
+      if ( *address ) {
+	hostent = gethostbyname(address);
+	if ( !hostent || hostent->h_addrtype != AF_INET ) {
+	  syslog(LOG_ERR, "cannot resolve local bind address: %s", address);
+	  exit(EX_NOINPUT);
+	}
+	memcpy(&bindaddr.sin_addr, hostent->h_addr, hostent->h_length);
+      } else {
+	/* Default to using INADDR_ANY */
+      }
+    
+      if ( portptr && *portptr ) {
+	servent = getservbyname(portptr, "udp");
+	if ( servent ) {
+	  bindaddr.sin_port = servent->s_port;
+	} else if ( (port = strtoul(portptr, &eportptr, 0)) && !*eportptr ) {
+	  bindaddr.sin_port = htons(port);
+	} else if ( !strcmp(portptr, "tftp") ) {
+	  /* It's TFTP, we're OK */
+	} else {
+	  syslog(LOG_ERR, "cannot resolve local bind port: %s", portptr);
+	  exit(EX_NOINPUT);
+	}
+      }
+    }
+
+    if (bind(fd, (struct sockaddr *)&bindaddr, sizeof bindaddr) < 0) {
+      syslog(LOG_ERR, "cannot bind to local socket: %m");
+      exit(EX_OSERR);
+    }
+
+    /* Daemonize this process */
+    {
+      int f = fork();
+      if ( f > 0 )
+	exit(0);
+      if ( f < 0 ) {
+	syslog(LOG_ERR, "cannot fork: %m");
+	exit(EX_OSERR);
+      }
+      close(0); close(1); close(2);
+#ifdef HAVE_SETSID
+      setsid();
+#endif
+    }
+  } else {
+    /* 0 is our socket descriptor */
+    close(1); close(2);
+  }
+
   /* This means we don't want to wait() for children */
   set_signal(SIGCHLD, SIG_IGN, SA_NOCLDSTOP);
   
@@ -296,25 +387,48 @@ main(int argc, char **argv)
     struct timeval tv_timeout;
     int rv;
     
+    if ( caught_sighup ) {
+      caught_sighup = 0;
+      if ( listen ) {
+#ifdef HAVE_REGEX
+	if ( rewrite_file ) {
+	  freerules(rewrite_rules);
+	  rewrite_rules = read_remap_rules(rewrite_file);
+	}
+#endif
+      } else {
+	/* Return to inetd for respawn */
+	exit(0);
+      }
+    }
+    
     FD_ZERO(&readset);
     FD_SET(fd, &readset);
     tv_timeout.tv_sec = timeout;
     tv_timeout.tv_usec = 0;
     
-    if ( caught_sighup )
-      exit(0);		/* Return to inetd for respawn */
-    
-    rv = select(fd+1, &readset, NULL, NULL, &tv_timeout);
+    /* Never time out if we're in listen mode */
+    rv = select(fd+1, &readset, NULL, NULL, listen ? NULL : &tv_timeout);
     if ( rv == -1 && errno == EINTR )
       continue;		/* Signal caught, reloop */
-    if ( rv <= 0 )
-      exit(0);		/* Timeout or error, return to inetd */
+    if ( rv == -1 ) {
+      syslog(LOG_ERR, "select loop: %m");
+      exit(EX_OSERR);
+    } else if ( rv == 0 ) {
+      exit(0);		/* Timeout, return to inetd */
+    }
     
     fromlen = sizeof (from);
     n = myrecvfrom(fd, buf, sizeof (buf), 0,
 		   (struct sockaddr *)&from, &fromlen,
 		   &myaddr);
-    
+
+    if ( listen && myaddr.sin_addr.s_addr == INADDR_ANY ) {
+      /* myrecvfrom() didn't capture the source address; but we might
+	 have bound to a specific address, if so we should use it */
+      memcpy(&myaddr.sin_addr, &bindaddr.sin_addr, sizeof bindaddr.sin_addr);
+    }
+
     if (n < 0) {
       syslog(LOG_ERR, "recvfrom: %m");
       exit(EX_IOERR);
@@ -334,6 +448,9 @@ main(int argc, char **argv)
   
   /* Child process: handle the actual request here */
   
+  /* Ignore SIGHUP */
+  set_signal(SIGHUP, SIG_IGN, 0);
+  
 #ifdef HAVE_TCPWRAPPERS
   /* Verify if this was a legal request for us.  This has to be
      done before the chroot, while /etc is still accessible. */
@@ -354,7 +471,19 @@ main(int argc, char **argv)
 	   inet_ntoa(from.sin_addr));
   }
 #endif
+
+  /* Close file descriptors we don't need */
+  close(fd);
   
+  /* Get a socket.  This has to be done before the chroot(), since
+     some systems require access to /dev to create a socket. */
+  
+  peer = socket(AF_INET, SOCK_DGRAM, 0);
+  if (peer < 0) {
+    syslog(LOG_ERR, "socket: %m");
+    exit(EX_IOERR);
+  }
+
   /* Chroot and drop privileges */
   
   if (secure && chroot(".")) {
@@ -381,30 +510,18 @@ main(int argc, char **argv)
     exit(EX_OSERR);
   }
   
-  /* Ignore SIGHUP */
-  set_signal(SIGHUP, SIG_IGN, 0);
-  
-  /* Close file descriptors we don't need */
-  close(fd);
-  close(1);
-  
   /* Other basic setup */
   from.sin_family = AF_INET;
   alarm(0);
   
   /* Process the request... */
   
-  peer = socket(AF_INET, SOCK_DGRAM, 0);
-  if (peer < 0) {
-    syslog(LOG_ERR, "socket: %m");
-    exit(EX_IOERR);
-  }
   myaddr.sin_port = htons(0); /* We want a new local port */
-  if (bind(peer, (struct sockaddr *)&myaddr, sizeof (myaddr)) < 0) {
+  if (bind(peer, (struct sockaddr *)&myaddr, sizeof myaddr) < 0) {
     syslog(LOG_ERR, "bind: %m");
     exit(EX_IOERR);
   }
-  if (connect(peer, (struct sockaddr *)&from, sizeof(from)) < 0) {
+  if (connect(peer, (struct sockaddr *)&from, sizeof from) < 0) {
     syslog(LOG_ERR, "connect: %m");
     exit(EX_IOERR);
   }
@@ -421,7 +538,7 @@ void	sendfile(struct formats *, struct tftphdr *, int);
 void	recvfile(struct formats *, struct tftphdr *, int);
 
 struct formats {
-  char	*f_mode;
+  const char *f_mode;
   char	*(*f_rewrite)(char *, int);
   int	(*f_validate)(char *, int, struct formats *);
   void	(*f_send)(struct formats *, struct tftphdr *, int);
@@ -521,7 +638,7 @@ tftp(struct tftphdr *tp, int size)
     else
       (*pf->f_send)(pf, NULL, 0);
   }
-  exit(EX_SOFTWARE);		/* We should never get here */
+  exit(0);			/* Request completed */
 }
 
 static int blksize_set;
@@ -670,6 +787,8 @@ rewrite_access(char *filename, int mode)
     char *newname = rewrite_string(filename, rewrite_rules, mode != RRQ);
     filename = newname;
   }
+#else
+  (void)mode;			/* Suppress unused warning */
 #endif
   return filename;
 }
@@ -690,8 +809,10 @@ int
 validate_access(char *filename, int mode, struct formats *pf)
 {
   struct stat stbuf;
+  int   i, len;
   int	fd, wmode;
-  char *cp, **dirp;
+  char *cp;
+  const char **dirp;
   
   tsize_ok = 0;
   
@@ -702,9 +823,13 @@ validate_access(char *filename, int mode, struct formats *pf)
      * prevent tricksters from getting around the directory
      * restrictions
      */
-    for (cp = filename + 1; *cp; cp++)
-      if(*cp == '.' && strncmp(cp-1, "/../", 4) == 0)
+    len = strlen(filename);
+    for ( i = 1 ; i < len-3 ; i++ ) {
+      cp = filename + i;
+      if ( *cp == '.' && memcmp(cp-1, "/../", 4) == 0)
 	return(EACCESS);
+    }
+
     for (dirp = dirs; *dirp; dirp++)
       if (strncmp(filename, *dirp, strlen(*dirp)) == 0)
 	break;
@@ -969,7 +1094,7 @@ recvfile(struct formats *pf, struct tftphdr *oap, int oacklen)
 
 struct errmsg {
   int	e_code;
-  char	*e_msg;
+  const char *e_msg;
 } errmsgs[] = {
   { EUNDEF,	"Undefined error code" },
   { ENOTFOUND,	"File not found" },
